/***************************************************************************
 ** This file is part of the lirc-0.5.4 package ****************************
 ** LIRC - Linux Infrared Remote Control ***********************************
 ***************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


/****************************************************************************
 ** irexec.c ****************************************************************
 ****************************************************************************
 *
 * irexec  - execute programs according to the pressed remote control buttons
 *
 * Copyright (C) 1998 Trent Piepho <xyzzy@u.washington.edu>
 * Copyright (C) 1998 Christoph Bartelmus <columbus@hit.handshake.de>
 *
 */

#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lirc_client.h"

char *progname;

int main(int argc, char *argv[])
{
	struct lirc_config *config;

	progname=argv[0];
	if(argc>2)
	{
		fprintf(stderr,"Usage: %s <config file>\n",progname);
		exit(EXIT_FAILURE);
	}
	if(lirc_init("irexec")==-1) exit(EXIT_FAILURE);

	if(lirc_readconfig(argc==2 ? argv[1]:NULL,&config,NULL)==0)
	{
		char *ir;
		char *c;

		while((ir=lirc_nextir())!=NULL)
		{
			while((c=lirc_ir2char(config,ir))!=NULL)
			{
#ifdef DEBUG
				printf("Execing command \"%s\"\n",c);
#endif
				system(c);
			}
			free(ir);
		}
		lirc_freeconfig(config);
	}

	lirc_deinit();
	exit(EXIT_SUCCESS);
}
